﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Threading;

namespace CatalexLib
{
    public enum CatalexCmd
    {
        CMD_NEXT_SONG = 0X01,
        CMD_PREV_SONG = 0X02,
        //CMD_PLAY_W_INDEX = 0X03,
        CMD_VOLUME_UP = 0X04,
        CMD_VOLUME_DOWN = 0X05,
        CMD_SET_VOLUME = 0X06,
        CMD_PLAY_W_INDEX = 0X08,
        CMD_SEL_DEV = 0X09,
        CMD_SINGLE_CYCLE = 0X19,
        CMD_SLEEP_MODE = 0X0A,
        CMD_WAKE_UP = 0X0B,
        CMD_RESET = 0X0C,
        CMD_PLAY = 0X0D,
        CMD_PAUSE = 0X0E,
        CMD_PLAY_FOLDER_FILE = 0X0F,
        CMD_STOP_PLAY = 0X16,
        CMD_SET_SINGLE_CYCLE = 0X19,
        CMD_SET_DAC = 0X1A,
        CMD_GROUP_DISPLAY = 0X21,
        CMD_PLAY_W_VOL = 0X22
    }
    public enum CatalexQueryCmd
    {
        STATUS = 0X42,
        TOTAL_FILES = 0X48,
        CURRENT_TRACK = 0X4C,
        FOLDER_FILE_COUNT = 0X4E,
        FOLDER_COUNT = 0X4F
    }

    public class CatalexBoard : IDisposable
    {
        internal SerialPort _serialPort;
        public delegate void NewMessageEventHandler(String msg);
        public event NewMessageEventHandler NewMessage;

        /* kind of a waste of energy to fill with the same thing over and over, so just init
        cmd_buff[0] = 0x7e; //starting byte
        cmd_buff[1] = 0xff; //version
        cmd_buff[2] = 0x06; //the number of bytes of the command without starting byte and ending byte
        --real digits here--
        cmd_buff[7] = 0xef; //ending byte
        */
        private byte[] cmd_buff = new byte[8] { 0x7e, 0xff, 0x06, 0, 0, 0, 0, 0xef };

        //private byte[] query_buff = new byte[18] { 0x7e, 0xff, 0x06, 0, 0, 0, 0, 0xfe, 0, 0xef, 0x7e, 0xff, 0x06, 0, 0, 0, 0, 0xef };
        private byte[] query_buff = new byte[10] { 0x7e, 0xff, 0x06, 0, 0, 0, 0, 0xfe, 0, 0xef };

        public CatalexBoard(string serialPortName)
        {
            this._serialPort = new SerialPort(serialPortName);
            _serialPort.BaudRate = 9600;
            _serialPort.Parity = Parity.None;
            _serialPort.DataBits = 8;
            _serialPort.StopBits = StopBits.One;
            _serialPort.NewLine = "\n";
        }
        public void setup()
        {
            //mySerial.begin(9600);
            Thread.Sleep(500);//Wait chip initialization is complete
            sendCommand(CatalexCmd.CMD_SEL_DEV, 0,0X02, false);//select the TF card  
            //Thread.Sleep(200);//wait for 200ms
            //sendCommand((byte)CatalexCmd.CMD_PLAY_W_VOL, (short)0X0F01, true);//play the first song with volume 15 class
        }
        //serial plumbing
        public void Start()
        {
            try
            {
                if (_serialPort != null && !this._serialPort.IsOpen)
                {
                    this._serialPort.DataReceived += new SerialDataReceivedEventHandler(_serialPort_DataReceived);
                    this._serialPort.Open();
                    setup();
                }
            }
            catch (Exception ex)
            {
                //Logging.ErrorException(string.Format("Serial Port Open failed"), ex);
                throw;
            }
        }
        void _serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                this._serialPort.DataReceived -= new SerialDataReceivedEventHandler(_serialPort_DataReceived);
                System.Threading.Thread.Sleep(100); //pause to let buffer fill
                string data = string.Empty;
                byte[] buffer = new byte[_serialPort.BytesToRead];
                int rcvdBytes = _serialPort.Read(buffer, 0, buffer.Length);
                data = "RECEIVED: " + BitConverter.ToString(buffer) + "\r\n";
                if (string.IsNullOrEmpty(data))
                {
                    this._serialPort.DataReceived += new SerialDataReceivedEventHandler(_serialPort_DataReceived);
                    return;
                }
                if (NewMessage != null)//If there is someone waiting for this event to be fired
                {
                    NewMessage(data); //Fire the event, 
                }
                this._serialPort.DataReceived += new SerialDataReceivedEventHandler(_serialPort_DataReceived);
            }
            catch (Exception)
            {

                //throw;
            }
            //try
            //{
            //    byte[] buffer = new byte[_serialPort.BytesToRead];
            //    int rcvdBytes = _serialPort.Read(buffer, 0, buffer.Length);
            //    lock (locker)
            //    {
            //        _receivedBytes.AddRange(buffer);
            //    }
            //    if (NewMessage != null)//If there is someone waiting for this event to be fired
            //    {
            //        NewMessage(Encoding.ASCII.GetString(_receivedBytes.ToArray())); //Fire the event, 
            //    }

            //    //Do the more interesting handling of the receivedBytes list here.
            //    _receivedBytes.Clear();
            //}
            //catch (Exception ex)
            //{
            //    System.Diagnostics.Debug.WriteLine(ex.ToString());
            //    //put other, more interesting error handling here.
            //}
        }

        public virtual void Stop()
        {
            if (_serialPort != null && this._serialPort.IsOpen)
            {
                this._serialPort.DataReceived -= new SerialDataReceivedEventHandler(_serialPort_DataReceived);
                while (!(_serialPort.BytesToRead == 0 && _serialPort.BytesToWrite == 0))
                {
                    _serialPort.DiscardInBuffer();
                    _serialPort.DiscardOutBuffer();
                }
                this._serialPort.Close();
            }
        }

        public void Dispose()
        {
            if (this._serialPort != null && this._serialPort.IsOpen)
            {
                this._serialPort.Close();
            }
            if (this._serialPort != null)
            {
                this._serialPort.DataReceived -= new SerialDataReceivedEventHandler(_serialPort_DataReceived);
                this._serialPort.Dispose();
                this._serialPort = null;
            }

            GC.SuppressFinalize(this);
        }

        public void sendCommand(CatalexCmd command, byte HiData,byte LowData, bool feedback)
        {
            cmd_buff[3] = Convert.ToByte(command); //
            cmd_buff[4] = (byte)(feedback ? 0x01 : 0x00);//0x00 = no feedback, 0x01 = feedback
            cmd_buff[5] = HiData;
            cmd_buff[6] = LowData;

            string data = command.ToString() + "\r\n" + "SENDING:  " + BitConverter.ToString(cmd_buff) + "\r\n";
            if (NewMessage != null)
            {
                NewMessage(data); //feedback to UI
            }

            this._serialPort.Write(cmd_buff, 0, cmd_buff.Length);
        }

        public void sendQCommand(CatalexQueryCmd command, byte fldr)
        {

            query_buff[0] = 0x7e; //starting byte
            query_buff[1] = 0xff; //version
            query_buff[2] = 0x06; //the number of bytes of the command without starting byte and ending byte
            query_buff[3] = Convert.ToByte(command);
            query_buff[4] = 0x00;
            query_buff[5] = 0x00;
            query_buff[6] = fldr;
            query_buff[7] = 0xfe;
            query_buff[8] = (byte)(0xfb - Convert.ToByte(command) - fldr);
            query_buff[9] = 0xef;
            //query_buff[10] = 0x7e;
            //query_buff[11] = 0xff;
            //query_buff[12] = 0x06;
            //query_buff[13] = Convert.ToByte(command);
            //query_buff[14] = 0x00;
            //query_buff[15] = 0x00;
            //query_buff[16] = fldr;
            //query_buff[17] = 0xef; //ending byte

            string data = command.ToString() + "\r\n" + "SENDING:  " + BitConverter.ToString(query_buff) + "\r\n";
            if (NewMessage != null)
            {
                NewMessage(data); //feedback to UI
            }

            this._serialPort.Write(query_buff, 0, 10);
        }
        public void test1()
        {
            //byte[] b = new byte[10] { 0x7E, 0xFF, 0x06, 0x0F, 0x00, 0x01, 0x02, 0xFE, 0xE9, 0xEF };
            //7E FF 06 0F 00 01 02 FE E9 EF
            //7E FF 06 4E 00 00 01 FE AC EF query 01 tracks total number of queries folder
            byte[] b = new byte[8] { 0x7E, 0xFF, 0x06, 0x4E, 0x00, 0x00, 0x01, 0xEF };
            //7E FF 06 4E 00 00 01 FE AC EF
            this._serialPort.Write(b, 0, b.Length);
        }

    }
}
